;; Print Emacs buffer on printer or buffer (using nroff, troff or pr).
;; Copyright (C) 1988 Ronald Florence (ron@mlfarm)
;; 	added optional flag for tbl, neqn, col to proof-to-buffer (1/24/89)
;;	added clean-file, col -b for proof-buffer (2/19/89)
;;	added troff-buffer, tproof-buffer, 3/28/89

(defun lp-buffer (&optional copies)
  "Print buffer contents with Unix command `pr | lp'.
Optional prefix argument specifies number of copies."
  (interactive "P")
  (prprint-region (point-min) (point-max) copies))

(defun lp-region (start end &optional copies)
  "Print region contents with Unix command `pr | lp'.
Optional prefix argument specifies number of copies."
  (interactive "r\nP")
  (prprint-region start end copies))

(defun print-buffer (&optional copies)
  "Print buffer contents after formatting with `nroff -cm -rN2 -rO9'.
Optional prefix argument specifies number of copies."
  (interactive "P")
  (nroff-to-printer-region (point-min) (point-max) copies))

(defun print-region (start end &optional copies)
  "Print region contents after formatting with `nroff -cm -rN2 -rO9'.
Optional prefix argument specifies number of copies."
  (interactive "r\nP")
  (nroff-to-printer-region start end copies))

(defun proof-buffer (&optional flag)
  "Proof buffer contents with `nroff -cm -rN2 -rO9' in another buffer.
Optional prefix argument invokes `tbl', `neqn' and `col' filters."
  (interactive "P")
  (nroff-to-buffer-region (point-min) (point-max) flag))

(defun proof-region (start end &optional flag)
  "Proof region contents with `nroff -cm -rN2 -rO9' in another buffer.
Optional prefix argument invokes `tbl', `neqn' and `col' filters."
  (interactive "r\nP")
  (nroff-to-buffer-region start end flag))

(defun clean-file (name)
  "Clean nroff junk from file and display in a buffer."
  (interactive "fClean file: ")
  (with-output-to-temp-buffer (file-name-nondirectory name)
    (save-excursion      
      (message "Cleaning...")
      (call-process "col" name standard-output t "-b")
      (message ""))))

(defun troff-buffer (&optional copies)
  "Typeset region contents after formatting with `troff -mm -rN2'.
Optional prefix argument specifies number of copies."
  (interactive "P")
  (let ((buffer (get-buffer-create "*printer command output*")))
    (save-excursion
      (set-buffer buffer)
      (erase-buffer))
    (message "Spooling...")
    (if (null copies) (setq copies "1"))
    (call-process-region (point-min) (point-max) "emacstroff" 
			 nil buffer nil  
			 (concat "-n" copies))
    (if (> (buffer-size) 0)
	(progn
	  (set-buffer buffer)
	  (goto-char (point-min))
	  (end-of-line)
	  (message "%s" (buffer-substring 1 (point))))
      (message "(Printer command failed)")))
  (kill-buffer "*printer command output*"))

(defun prprint-region (start end &optional copies)
  (let ((buffer (get-buffer-create "*printer command output*")))
    (save-excursion
      (set-buffer buffer)
      (erase-buffer))
    (message "Spooling...")
    (if (/= tab-width 8)
	(let ((oldbuf (current-buffer)))
	  (set-buffer (get-buffer-create " *spool temp*"))
	  (widen) (erase-buffer)
	  (insert-buffer-substring oldbuf)
	  (setq tab-width width)
	  (untabify (point-min) (point-max))
	  (setq start (point-min) end (point-max))))
    (if (null copies) (setq copies "1"))
    (call-process-region start end "prprint"
			 nil buffer nil
			 (concat "-h " (buffer-name)) (concat "-n" copies))
    (if (> (buffer-size) 0)
	(progn
	  (set-buffer buffer)
	  (goto-char (point-min))
	  (end-of-line)
	  (message "%s" (buffer-substring 1 (point))))
      (message "(Printer command failed)")))
  (kill-buffer "*printer command output*"))

(defun nroff-to-printer-region (start end &optional copies)
  (let ((buffer (get-buffer-create "*printer command output*")))
    (save-excursion
      (set-buffer buffer)
      (erase-buffer))
    (message "Spooling...")
    (if (null copies) (setq copies "1"))
    (call-process-region start end "emacsprint" 
			 nil buffer nil  
			 (concat "-n" copies))
    (if (> (buffer-size) 0)
	(progn
	  (set-buffer buffer)
	  (goto-char (point-min))
	  (end-of-line)
	  (message "%s" (buffer-substring 1 (point))))
      (message "(Printer command failed)")))
  (kill-buffer "*printer command output*"))

(defun tproof-buffer ()
  "Rough proof buffer contents after formatting with `troff -mm -a -rN2'."
  (interactive)
  (with-output-to-temp-buffer "*Proof Output*"
    (save-excursion
      (message "Proofing...")
	(progn
	  (call-process-region (point-min) (point-max) "troff" 
			       nil standard-output nil
			       "-mm" "-a" "-rN2")
	  (set-buffer standard-output)
      (set-buffer-modified-p nil)
      (message "")))))

(defun nroff-to-buffer-region (start end &optional flag)
  (with-output-to-temp-buffer "*Proof Output*"
    (save-excursion
      (setq start (point-min) end (point-max))
      (message "Proofing...")
      (if flag
	  (call-process-region start end "colformat" 
			       nil standard-output nil)
	(progn
	  (call-process-region start end "nroff" 
			       nil standard-output nil
			       "-cm" "-rN2" "-rO9")
	  (set-buffer standard-output)
	  (message "Cleaning proof output...") 
	  (zap-nroff-crap)))
      (set-buffer-modified-p nil)
      (message ""))))

(defun zap-nroff-crap ()
  (interactive "*")
  (goto-char (point-min))
  (while (search-forward "\b" nil t)
    (let* ((preceding (char-after (- (point) 2)))
	   (following (following-char)))
      	    ;; x\bx
      (cond ((= preceding following)	
	     (delete-char -2))
	    ;; _\b
	    ((= preceding ?\_)		
	     (delete-char -2))
	    ;; \b_
	    ((= following ?\_)		
	     (delete-region (1- (point)) (1+ (point)))))))
  ;; expand ^G lines
  (goto-char (point-min))
  (while (search-forward "\C-g" nil t)	
    (delete-char -2)
    (while (not (eolp))
      (insert " ")
      (forward-char 1)))
  ;; zap Esc-8 & Esc-9 vertical motions
  (goto-char (point-min))
  (while (search-forward "\e" nil t)
    (if (or (= (following-char) ?8) (= (following-char) ?9))
	    (delete-region (1+ (point)) (1- (point))))))
